@@ -2,8 +2,11 @@ use std::{pin::Pin, sync::Arc};
 
 use crate::{
     os::task::{thread::WasiThreadRunGuard, TaskJoinHandle},
-    runtime::task_manager::{
-        TaskWasm, TaskWasmRecycle, TaskWasmRecycleProperties, TaskWasmRunProperties,
+    runtime::{
+        task_manager::{
+            TaskWasm, TaskWasmRecycle, TaskWasmRecycleProperties, TaskWasmRunProperties,
+        },
+        TaintReason,
     },
     syscalls::rewind_ext,
     RewindState, SpawnError, WasiError, WasiRuntimeError,
@@ -188,6 +191,7 @@ fn call_module(
     let pid = env.pid();
     let tasks = env.tasks().clone();
     handle.thread.set_status_running();
+    let runtime = env.runtime.clone();
 
     // If we need to rewind then do so
     if let Some((rewind_state, rewind_result)) = rewind_state {
@@ -195,7 +199,7 @@ fn call_module(
         if rewind_state.is_64bit {
             let res = rewind_ext::<Memory64>(
                 &mut ctx,
-                rewind_state.memory_stack,
+                Some(rewind_state.memory_stack),
                 rewind_state.rewind_stack,
                 rewind_state.store_data,
                 rewind_result,
@@ -208,7 +212,7 @@ fn call_module(
         } else {
             let res = rewind_ext::<Memory32>(
                 &mut ctx,
-                rewind_state.memory_stack,
+                Some(rewind_state.memory_stack),
                 rewind_state.rewind_stack,
                 rewind_state.store_data,
                 rewind_result,
@@ -237,7 +241,10 @@ fn call_module(
         if let Err(err) = call_ret {
             match err.downcast::<WasiError>() {
                 Ok(WasiError::Exit(code)) if code.is_success() => Ok(Errno::Success),
-                Ok(err @ WasiError::Exit(_)) => Err(err.into()),
+                Ok(WasiError::Exit(code)) => {
+                    runtime.on_taint(TaintReason::NonZeroExitCode(code));
+                    Err(WasiError::Exit(code).into())
+                }
                 Ok(WasiError::DeepSleep(deep)) => {
                     // Create the callback that will be invoked when the thread respawns after a deep sleep
                     let rewind = deep.rewind;
@@ -264,9 +271,13 @@ fn call_module(
                 }
                 Ok(WasiError::UnknownWasiVersion) => {
                     debug!("failed as wasi version is unknown",);
+                    runtime.on_taint(TaintReason::UnknownWasiVersion);
                     Ok(Errno::Noexec)
                 }
-                Err(err) => Err(WasiRuntimeError::from(err)),
+                Err(err) => {
+                    runtime.on_taint(TaintReason::RuntimeError(err.clone()));
+                    Err(WasiRuntimeError::from(err))
+                }
             }
         } else {
             Ok(Errno::Success)
