@@ -55,9 +55,6 @@ mod state;
 mod syscalls;
 mod utils;
 
-/// WAI based bindings.
-mod bindings;
-
 use std::sync::Arc;
 
 #[allow(unused_imports)]
@@ -685,7 +682,7 @@ fn stub_initializer(
 // TODO: split function into two variants, one for JS and one for sys.
 // (this will make code less messy)
 fn import_object_for_all_wasi_versions(
-    module: &wasmer::Module,
+    _module: &wasmer::Module,
     store: &mut impl AsStoreMut,
     env: &FunctionEnv<WasiEnv>,
 ) -> (Imports, ModuleInitializer) {
@@ -705,31 +702,7 @@ fn import_object_for_all_wasi_versions(
         "wasix_64v1" => exports_wasix_64v1,
     };
 
-    // TODO: clean this up!
-    cfg_if::cfg_if! {
-        if #[cfg(feature = "sys")] {
-            // Check if the module needs http.
-
-            let has_canonical_realloc = module.exports().any(|t| t.name() == "canonical_abi_realloc");
-            let has_wasix_http_import = module.imports().any(|t| t.module() == "wasix_http_client_v1");
-
-            let init = if has_canonical_realloc && has_wasix_http_import {
-                let wenv = env.as_ref(store);
-                let http = crate::http::client_impl::WasixHttpClientImpl::new(wenv);
-                    crate::bindings::wasix_http_client_v1::add_to_imports(
-                        store,
-                        &mut imports,
-                        http,
-                    )
-            } else {
-                Box::new(stub_initializer) as ModuleInitializer
-            };
-        } else {
-            // Prevents unused warning.
-            let _ = module;
-            let init = Box::new(stub_initializer) as ModuleInitializer;
-        }
-    }
+    let init = Box::new(stub_initializer) as ModuleInitializer;
 
     (imports, init)
 }
@@ -799,3 +772,22 @@ pub(crate) fn block_in_place<Ret>(thunk: impl FnOnce() -> Ret) -> Ret {
         }
     }
 }
+
+/// Spawns a new blocking task that runs the provided closure.
+///
+/// The closure is executed on a separate thread, allowing it to perform blocking operations
+/// without blocking the main thread. The closure is wrapped in a `Future` that resolves to the
+/// result of the closure's execution.
+pub(crate) async fn spawn_blocking<F, R>(f: F) -> Result<R, tokio::task::JoinError>
+where
+    F: FnOnce() -> R + Send + 'static,
+    R: Send + 'static,
+{
+    cfg_if::cfg_if! {
+        if #[cfg(target_arch = "wasm32")] {
+            Ok(block_in_place(f))
+        } else {
+            tokio::task::spawn_blocking(f).await
+        }
+    }
+}
