@@ -4,16 +4,17 @@ use std::{path::PathBuf, sync::Arc};
 
 use anyhow::{Context, Error};
 use tracing::Instrument;
-use virtual_fs::{ArcBoxFile, TmpFileSystem, VirtualFile};
-use wasmer::Module;
+use virtual_fs::{ArcBoxFile, FileSystem, TmpFileSystem, VirtualFile};
+use wasmer::{Extern, Module};
 use webc::metadata::{annotations::Wasi, Command};
 
 use crate::{
     bin_factory::BinaryPackage,
     capabilities::Capabilities,
-    runners::{wasi_common::CommonWasiOptions, MappedDirectory},
-    runtime::task_manager::VirtualTaskManagerExt,
-    Runtime, WasiEnvBuilder, WasiRuntimeError,
+    journal::{DynJournal, SnapshotTrigger},
+    runners::{wasi_common::CommonWasiOptions, MappedDirectory, MountedDirectory},
+    runtime::{module_cache::ModuleHash, task_manager::VirtualTaskManagerExt},
+    Runtime, WasiEnvBuilder, WasiError, WasiRuntimeError,
 };
 
 use super::wasi_common::MappedCommand;
@@ -38,46 +39,22 @@ impl WasiRunner {
     }
 
     /// Builder method to provide CLI args to the runner
-    pub fn with_args<A, S>(mut self, args: A) -> Self
-    where
-        A: IntoIterator<Item = S>,
-        S: Into<String>,
-    {
-        self.set_args(args);
-        self
-    }
-
-    /// Set the CLI args
-    pub fn set_args<A, S>(&mut self, args: A)
+    pub fn with_args<A, S>(&mut self, args: A) -> &mut Self
     where
         A: IntoIterator<Item = S>,
         S: Into<String>,
     {
         self.wasi.args = args.into_iter().map(|s| s.into()).collect();
+        self
     }
 
     /// Builder method to provide environment variables to the runner.
     pub fn with_env(mut self, key: impl Into<String>, value: impl Into<String>) -> Self {
-        self.set_env(key, value);
-        self
-    }
-
-    /// Provide environment variables to the runner.
-    pub fn set_env(&mut self, key: impl Into<String>, value: impl Into<String>) {
         self.wasi.env.insert(key.into(), value.into());
-    }
-
-    pub fn with_envs<I, K, V>(mut self, envs: I) -> Self
-    where
-        I: IntoIterator<Item = (K, V)>,
-        K: Into<String>,
-        V: Into<String>,
-    {
-        self.set_envs(envs);
         self
     }
 
-    pub fn set_envs<I, K, V>(&mut self, envs: I)
+    pub fn with_envs<I, K, V>(&mut self, envs: I) -> &mut Self
     where
         I: IntoIterator<Item = (K, V)>,
         K: Into<String>,
@@ -86,51 +63,51 @@ impl WasiRunner {
         for (key, value) in envs {
             self.wasi.env.insert(key.into(), value.into());
         }
-    }
-
-    pub fn with_forward_host_env(mut self, forward: bool) -> Self {
-        self.set_forward_host_env(forward);
         self
     }
 
-    pub fn set_forward_host_env(&mut self, forward: bool) {
+    pub fn with_forward_host_env(&mut self, forward: bool) -> &mut Self {
         self.wasi.forward_host_env = forward;
+        self
     }
 
-    pub fn with_mapped_directories<I, D>(mut self, dirs: I) -> Self
+    pub fn with_mapped_directories<I, D>(&mut self, dirs: I) -> &mut Self
     where
         I: IntoIterator<Item = D>,
         D: Into<MappedDirectory>,
     {
-        self.wasi
-            .mapped_dirs
-            .extend(dirs.into_iter().map(|d| d.into()));
+        self.with_mounted_directories(dirs.into_iter().map(Into::into).map(MountedDirectory::from))
+    }
+
+    pub fn with_mounted_directories<I, D>(&mut self, dirs: I) -> &mut Self
+    where
+        I: IntoIterator<Item = D>,
+        D: Into<MountedDirectory>,
+    {
+        self.wasi.mounts.extend(dirs.into_iter().map(Into::into));
         self
     }
 
-    pub fn set_current_dir(&mut self, dir: impl Into<PathBuf>) {
-        self.wasi.current_dir = Some(dir.into());
+    /// Mount a [`FileSystem`] instance at a particular location.
+    pub fn with_mount(&mut self, dest: String, fs: Arc<dyn FileSystem + Send + Sync>) -> &mut Self {
+        self.wasi.mounts.push(MountedDirectory { guest: dest, fs });
+        self
     }
 
-    pub fn with_current_dir(mut self, dir: impl Into<PathBuf>) -> Self {
-        self.set_current_dir(dir);
+    /// Override the directory the WASIX instance will start in.
+    pub fn with_current_dir(&mut self, dir: impl Into<PathBuf>) -> &mut Self {
+        self.wasi.current_dir = Some(dir.into());
         self
     }
 
     /// Add a package that should be available to the instance at runtime.
-    pub fn add_injected_package(&mut self, pkg: BinaryPackage) -> &mut Self {
+    pub fn with_injected_package(&mut self, pkg: BinaryPackage) -> &mut Self {
         self.wasi.injected_packages.push(pkg);
         self
     }
 
-    /// Add a package that should be available to the instance at runtime.
-    pub fn with_injected_package(mut self, pkg: BinaryPackage) -> Self {
-        self.add_injected_package(pkg);
-        self
-    }
-
     /// Add packages that should be available to the instance at runtime.
-    pub fn add_injected_packages(
+    pub fn with_injected_packages(
         &mut self,
         packages: impl IntoIterator<Item = BinaryPackage>,
     ) -> &mut Self {
@@ -138,40 +115,23 @@ impl WasiRunner {
         self
     }
 
-    /// Add packages that should be available to the instance at runtime.
-    pub fn with_injected_packages(
-        mut self,
-        packages: impl IntoIterator<Item = BinaryPackage>,
-    ) -> Self {
-        self.add_injected_packages(packages);
-        self
-    }
-
-    pub fn add_mapped_host_command(&mut self, alias: impl Into<String>, target: impl Into<String>) {
+    pub fn with_mapped_host_command(
+        &mut self,
+        alias: impl Into<String>,
+        target: impl Into<String>,
+    ) -> &mut Self {
         self.wasi.mapped_host_commands.push(MappedCommand {
             alias: alias.into(),
             target: target.into(),
         });
-    }
-
-    pub fn with_mapped_host_command(
-        mut self,
-        alias: impl Into<String>,
-        target: impl Into<String>,
-    ) -> Self {
-        self.add_mapped_host_command(alias, target);
         self
     }
 
-    pub fn add_mapped_host_commands(&mut self, commands: impl IntoIterator<Item = MappedCommand>) {
-        self.wasi.mapped_host_commands.extend(commands);
-    }
-
     pub fn with_mapped_host_commands(
-        mut self,
+        &mut self,
         commands: impl IntoIterator<Item = MappedCommand>,
-    ) -> Self {
-        self.add_mapped_host_commands(commands);
+    ) -> &mut Self {
+        self.wasi.mapped_host_commands.extend(commands);
         self
     }
 
@@ -179,42 +139,81 @@ impl WasiRunner {
         &mut self.wasi.capabilities
     }
 
-    pub fn with_capabilities(mut self, capabilities: Capabilities) -> Self {
-        self.set_capabilities(capabilities);
-        self
-    }
-
-    pub fn set_capabilities(&mut self, capabilities: Capabilities) {
+    pub fn with_capabilities(&mut self, capabilities: Capabilities) -> &mut Self {
         self.wasi.capabilities = capabilities;
-    }
-
-    pub fn with_stdin(mut self, stdin: Box<dyn VirtualFile + Send + Sync>) -> Self {
-        self.set_stdin(stdin);
         self
     }
 
-    pub fn set_stdin(&mut self, stdin: Box<dyn VirtualFile + Send + Sync>) -> &mut Self {
+    pub fn with_snapshot_trigger(&mut self, on: SnapshotTrigger) -> &mut Self {
+        self.wasi.snapshot_on.push(on);
+        self
+    }
+
+    pub fn with_default_snapshot_triggers(&mut self) -> &mut Self {
+        for on in crate::journal::DEFAULT_SNAPSHOT_TRIGGERS {
+            if !self.has_snapshot_trigger(on) {
+                self.with_snapshot_trigger(on);
+            }
+        }
+        self
+    }
+
+    pub fn has_snapshot_trigger(&self, on: SnapshotTrigger) -> bool {
+        self.wasi.snapshot_on.iter().any(|t| *t == on)
+    }
+
+    pub fn with_snapshot_interval(&mut self, period: std::time::Duration) -> &mut Self {
+        if !self.has_snapshot_trigger(SnapshotTrigger::PeriodicInterval) {
+            self.with_snapshot_trigger(SnapshotTrigger::PeriodicInterval);
+        }
+        self.wasi.snapshot_interval.replace(period);
+        self
+    }
+
+    pub fn with_journal(&mut self, journal: Arc<DynJournal>) -> &mut Self {
+        self.wasi.journals.push(journal);
+        self
+    }
+
+    pub fn with_stdin(&mut self, stdin: Box<dyn VirtualFile + Send + Sync>) -> &mut Self {
         self.stdin = Some(ArcBoxFile::new(stdin));
         self
     }
 
-    pub fn with_stdout(mut self, stdout: Box<dyn VirtualFile + Send + Sync>) -> Self {
-        self.set_stdout(stdout);
-        self
-    }
-
-    pub fn set_stdout(&mut self, stdout: Box<dyn VirtualFile + Send + Sync>) -> &mut Self {
+    pub fn with_stdout(&mut self, stdout: Box<dyn VirtualFile + Send + Sync>) -> &mut Self {
         self.stdout = Some(ArcBoxFile::new(stdout));
         self
     }
 
-    pub fn with_stderr(mut self, stderr: Box<dyn VirtualFile + Send + Sync>) -> Self {
-        self.set_stderr(stderr);
+    pub fn with_stderr(&mut self, stderr: Box<dyn VirtualFile + Send + Sync>) -> &mut Self {
+        self.stderr = Some(ArcBoxFile::new(stderr));
         self
     }
 
-    pub fn set_stderr(&mut self, stderr: Box<dyn VirtualFile + Send + Sync>) -> &mut Self {
-        self.stderr = Some(ArcBoxFile::new(stderr));
+    /// Add an item to the list of importable items provided to the instance.
+    pub fn with_import(
+        &mut self,
+        namespace: impl Into<String>,
+        name: impl Into<String>,
+        value: impl Into<Extern>,
+    ) -> &mut Self {
+        self.with_imports([((namespace, name), value)])
+    }
+
+    /// Add multiple import functions.
+    ///
+    /// This method will accept a [`&Imports`][wasmer::Imports] object.
+    pub fn with_imports<I, S1, S2, E>(&mut self, imports: I) -> &mut Self
+    where
+        I: IntoIterator<Item = ((S1, S2), E)>,
+        S1: Into<String>,
+        S2: Into<String>,
+        E: Into<Extern>,
+    {
+        let imports = imports
+            .into_iter()
+            .map(|((ns, n), e)| ((ns.into(), n.into()), e.into()));
+        self.wasi.additional_imports.extend(imports);
         self
     }
 
@@ -231,6 +230,7 @@ impl WasiRunner {
 
         let container_fs = if let Some(pkg) = pkg {
             builder.add_webc(pkg.clone());
+            builder.set_module_hash(pkg.hash());
             Some(Arc::clone(&pkg.webc_fs))
         } else {
             None
@@ -257,6 +257,7 @@ impl WasiRunner {
         runtime: Arc<dyn Runtime + Send + Sync>,
         program_name: &str,
         module: &Module,
+        module_hash: ModuleHash,
         asyncify: bool,
     ) -> Result<(), Error> {
         let wasi = webc::metadata::annotations::Wasi::new(program_name);
@@ -264,9 +265,9 @@ impl WasiRunner {
         let env = self.prepare_webc_env(program_name, &wasi, None, runtime, None)?;
 
         if asyncify {
-            env.run_with_store_async(module.clone(), store)?;
+            env.run_with_store_async(module.clone(), module_hash, store)?;
         } else {
-            env.run_with_store(module.clone(), &mut store)?;
+            env.run_with_store_ext(module.clone(), module_hash, &mut store)?;
         }
 
         Ok(())
@@ -295,11 +296,23 @@ impl crate::runners::Runner for WasiRunner {
             .annotation("wasi")?
             .unwrap_or_else(|| Wasi::new(command_name));
 
-        let env = self
+        #[allow(unused_mut)]
+        let mut env = self
             .prepare_webc_env(command_name, &wasi, Some(pkg), Arc::clone(&runtime), None)
-            .context("Unable to prepare the WASI environment")?
-            .build()?;
+            .context("Unable to prepare the WASI environment")?;
 
+        #[cfg(feature = "journal")]
+        {
+            for journal in self.wasi.journals.clone() {
+                env.add_journal(journal);
+            }
+
+            for snapshot_trigger in self.wasi.snapshot_on.iter().cloned() {
+                env.add_snapshot_trigger(snapshot_trigger);
+            }
+        }
+
+        let env = env.build()?;
         let store = runtime.new_store();
 
         let command_name = command_name.to_string();
@@ -316,11 +329,55 @@ impl crate::runners::Runner for WasiRunner {
                 task_handle
                     .wait_finished()
                     .await
-                    .map_err(|err| Arc::into_inner(err).expect("Error shouldn't be shared"))
+                    .map_err(|err| {
+                        // We do our best to recover the error
+                        let msg = err.to_string();
+                        let weak = Arc::downgrade(&err);
+                        Arc::into_inner(err).unwrap_or_else(|| {
+                            weak.upgrade()
+                                .map(|err| match err.as_ref() {
+                                    WasiRuntimeError::Init(a) => WasiRuntimeError::Init(a.clone()),
+                                    WasiRuntimeError::Export(a) => {
+                                        WasiRuntimeError::Export(a.clone())
+                                    }
+                                    WasiRuntimeError::Instantiation(a) => {
+                                        WasiRuntimeError::Instantiation(a.clone())
+                                    }
+                                    WasiRuntimeError::Wasi(WasiError::Exit(a)) => {
+                                        WasiRuntimeError::Wasi(WasiError::Exit(*a))
+                                    }
+                                    WasiRuntimeError::Wasi(WasiError::UnknownWasiVersion) => {
+                                        WasiRuntimeError::Wasi(WasiError::UnknownWasiVersion)
+                                    }
+                                    WasiRuntimeError::Wasi(WasiError::DeepSleep(_)) => {
+                                        WasiRuntimeError::Anyhow(Arc::new(anyhow::format_err!(
+                                            "deep-sleep"
+                                        )))
+                                    }
+                                    WasiRuntimeError::ControlPlane(a) => {
+                                        WasiRuntimeError::ControlPlane(a.clone())
+                                    }
+                                    WasiRuntimeError::Runtime(a) => {
+                                        WasiRuntimeError::Runtime(a.clone())
+                                    }
+                                    WasiRuntimeError::Thread(a) => {
+                                        WasiRuntimeError::Thread(a.clone())
+                                    }
+                                    WasiRuntimeError::Anyhow(a) => {
+                                        WasiRuntimeError::Anyhow(a.clone())
+                                    }
+                                })
+                                .unwrap_or_else(|| {
+                                    WasiRuntimeError::Anyhow(Arc::new(anyhow::format_err!(
+                                        "{}", msg
+                                    )))
+                                })
+                        })
+                    })
                     .context("Unable to wait for the process to exit")
             }
             .in_current_span(),
-        )?;
+        )??;
 
         if exit_code.raw() == 0 {
             Ok(())
