@@ -6,7 +6,10 @@ use crate::journal::JournalEffector;
 use crate::{
     capture_instance_snapshot,
     os::task::thread::WasiMemoryLayout,
-    runtime::task_manager::{TaskWasm, TaskWasmRunProperties},
+    runtime::{
+        task_manager::{TaskWasm, TaskWasmRunProperties},
+        TaintReason,
+    },
     syscalls::*,
     WasiThreadHandle,
 };
@@ -177,6 +180,9 @@ fn call_module<M: MemorySize>(
                     ret = if code.is_success() {
                         Errno::Success
                     } else {
+                        env.data(&store)
+                            .runtime
+                            .on_taint(TaintReason::NonZeroExitCode(code));
                         Errno::Noexec
                     };
                 }
@@ -186,10 +192,16 @@ fn call_module<M: MemorySize>(
                 }
                 Ok(WasiError::UnknownWasiVersion) => {
                     debug!("failed as wasi version is unknown",);
+                    env.data(&store)
+                        .runtime
+                        .on_taint(TaintReason::UnknownWasiVersion);
                     ret = Errno::Noexec;
                 }
                 Err(err) => {
                     debug!("failed with runtime error: {}", err);
+                    env.data(&store)
+                        .runtime
+                        .on_taint(TaintReason::RuntimeError(err));
                     ret = Errno::Noexec;
                 }
             }
@@ -208,7 +220,7 @@ fn call_module<M: MemorySize>(
         let mut ctx = ctx.env.clone().into_mut(&mut store);
         let res = rewind_ext::<M>(
             &mut ctx,
-            rewind_state.memory_stack,
+            Some(rewind_state.memory_stack),
             rewind_state.rewind_stack,
             rewind_state.store_data,
             Some(rewind_result),
